function [p_exact, p_approx, p_hist] = Schlogl_invMeasure(X0, Omega, N, theta)
%
% Compute the invariant measure of the Schlogl model with three different
% ways: (a) exactly, (b) Hangii approximation and (c) numerical (histogram)
% approximation.
%
%  ---- INPUT PARAMETERS ----
%   X0: initial number of molecules, [1x1]
%   Omega: volume of the system, [1x1]
%   N: maximum number of molecules, [1x1]
%   theta: rates of the Schlogl model, [1xd]
%
% ---- OUTPUT PARAMETERS ----
%   p_exact: exact invariane measure, [1xN+1]
%   p_approx: Hangii approximation of the invariane measure, [1xN+1]
%   p_hist: numerical approximation of the invariane measure, [1xN+1]
%
% (c) Math & Stat, Umass, Amherst, 2014

n = 0:N;
x = (1:N)/Omega;

% rates
k1A = theta(1);
k2 = theta(2);
k3B = theta(3);
k4 = theta(4);



% EXACT INVARIANT MEASURE
c_plus = k1A/Omega * n.*(n-1)/2 + k3B*Omega;
c_minus = k2/Omega^2 * n.*(n-1).*(n-2)/6 + k4*n;

p_exact = zeros(1, N);
p_exact(1)=1;

for i=1:N;
    p_exact(i+1) = p_exact(i) * c_plus(i)/c_minus(i+1);
end

p_exact = p_exact / sum(p_exact);



% HANGII APPROXIMATION OF INVARIANT MEASURE
phi0 = -x.*(1 + log((k1A/2*x.^2+k3B)./(k2/6*x.^3+k4*x))) + 2*sqrt(k4/(k2/6))*atan(x/sqrt(k4/(k2/6))) ...
     - 2*sqrt(k3B/(k1A/2))*atan(x/sqrt(k3B/(k1A/2)));

p_approx = 1 ./ sqrt((k1A/2*x.^2+k3B) .* (k2/6*x.^3+k4*x)) .* exp(-Omega*phi0);

p_approx = [0, p_approx / sum(p_approx)];



% HISTOGRAM (NUMERICAL) APPROXIMATION OF INVARIANT MEASURE
if nargout>2
NoJ = 500000; % number of jumps
T = zeros(NoJ, 1); % time
X = zeros(NoJ, 1); % number of X molecules
p_hist = zeros(1, N+1); % histogram of rates

StM = [1; -1; 1; -1]; % stoichiometric matrix (dim: #reactions x #species)
rt_tbl = Schlogl_rates_table(Omega, N); % table with the rates of the process

X(1) = X0; % initial number of molecules

% SSA
for i = 2:NoJ
    rn = rand(2,1); % two uniform random numbers, necessary for SSA
    
    rt = theta .* rt_tbl(X(i-1)+1, :); % rates of the process
    
    tot_rt = sum(rt); % total rate
    trp = cumsum(rt) / tot_rt; % transition probabilities
    
    dt = - log(rn(1))/tot_rt; % advance time
    T(i) = T(i-1) + dt;
    p_hist(X(i-1)+1) = p_hist(X(i-1)+1) + dt;

    idx = find(trp>rn(2), 1, 'first'); % find which reaction will occur
    
    X(i) = X(i-1) + StM(idx); % perform the reaction (i.e., update the number of molecules)
end

p_hist = p_hist / T(NoJ);
end

