function PFIM = comp_PFIM(X, data)
%
% Compute PFIM with input time series(species population only, no time) 
%
%  ---- INPUT PARAMETERS ----
%   X: Time series of the populations of chemical species 
%   data: data structure array
%
%   data.M: number of reactions, [1x1]
%   data.K: number of parameters, [1x1] 
%   data.X: current population of the system, [Nx1]
%	data.T: final time, [1x1]
%   data.cv_glpf: vector of the non-zero elements of the symbolic PFIM, [prop_lenx1]
%   data.prop_index: index of the non-zero elements of the symbolic PFIM, [prop_lenx1]
%   data.con: value of reaction constants, [Kx1]
%   data.stoichio: stoichiometric matrix, [Mx4]
%   data.cm_rea: connectivity matrix for the reactions, [Mx4]
%
% ---- OUTPUT PARAMETERS ----
%   PFIM: pathwise Fisher Information Matrix, [KxK]
%
% (c) Math & Stat, Umass, Amherst, 2014

% initialize population and time
[NoJ, N] = size(X);
data.X = X(1,:)';
t = 0;

% Initialize PFIM 
F = sparse(data.K,data.K);

% Rates initialization
c = zeros(data.M, 1);
for k = 1:data.M
    c(k) = reaction_rate(data, k);
end
c_cum = cumsum(c);
c0 = c_cum(end);

% iterate
for i = 2 : NoJ
    % check if there is nowhere to move
    if(~c0)
        fprintf('\n\nSystem reached a degenerate point. Exiting...\n\n');         
        return;
    end
        
    % compute waiting time
    dt = - log(rand)/c0;
    
    % update PFIM 
    F = update_PFIM(data, F, dt);
    
    if any(any(isnan(F)))
        error('FormatChk:eqChk', 'Nan generated in iteration %d.', i)
    end
    
    % update species populations
    data.X = X(i,:)';
    
    % update rates 
    for k = 1:data.M
        c(k) = reaction_rate(data, k);
    end
    c_cum = cumsum(c);
    c0 = c_cum(end);
 
    % update time
    t = t + dt;
    
    if t > data.T
        t = t - dt;
        break;
    end
end

PFIM = F/t; % normalization

