function RER = comp_RER(X, data)
%
% Compute a vector of RERs with input time series(species population only, no time)
% and perturbation matrix(each row represents a perturbation vector to all parameters)
%
%  ---- INPUT PARAMETERS ----
%   X: Time series of the populations of chemical species, [NoJ, N] 
%   data: data structure array
%
%   data.M: number of reactions, [1x1]
%   data.K: number of parameters, [1x1] 
%   data.X: current population of the system, [Nx1]
%	data.T: final time, [1x1]
%   data.con: value of reaction constants, [Kx1]
%   data.stoichio: stoichiometric matrix, [Mx4]
%   data.cm_rea: connectivity matrix for the reactions, [Mx4]
%   data.pert_vec: Matrix of positive and negative 10 percent perturbation for all parameter values, [NoP, K]
%
% ---- OUTPUT PARAMETERS ----
%   RER: a vector of RERs corresponds to the perturbation matrix, [NoPx1]
%
% (c) Math & Stat, Umass, Amherst, 2014

[NoP, K] = size(data.pert_vec);
[NoJ, N] = size(X);

% initialize population and time
data.X = X(1,:)';
t = 0;

% Initialize RER 
H = zeros(NoP, 1);

% Rates initialization
c = zeros(data.M, 1);
for k = 1:data.M
    c(k) = reaction_rate(data, k);
end
c_cum = cumsum(c);
c0 = c_cum(end);

% iterate
for i = 2 : NoJ
    % check if there is nowhere to move
    if(~c0)
        fprintf('\n\nSystem reached a degenerate point. Exiting...\n\n');         
        return;
    end
        
    % compute waiting time
    dt = - log(rand)/c0;  

    % update RER 
    for k = 1:NoP
        % for each perturbation vector, compute the perturbations
        veps = data.pert_vec(k,:)';
        
        % compute RER
        upRER = update_RER_per(data, dt, veps);              
        H(k) = H(k) + upRER;
        
    end
    
    % update species populations
    data.X = X(i,:)';
    
    % update rates 
    for k = 1:data.M
        c(k) = reaction_rate(data, k);
    end
    c_cum = cumsum(c);
    c0 = c_cum(end);
 
    % update time
    t = t + dt;
    
    if t > data.T
        t = t - dt;
        break;
    end
end

RER = H/t; % normalization

