function [RER, RER_exact, FIM, FIM_exact] = comp_RER_FIM_ssaSchlolg(X0, Omega, N, theta, epsilon)
%
% Computes the relative entropy rate as well the generalized Fisher
% information matrix for the Schlogl model using stochastic simulation
% algorithm (SSA).
%
%  ---- INPUT PARAMETERS ----
%   X0: initial configuration, [1x1]
%   Omega: volume of the system, [1x1]
%   N: maximum number of molecules, [1x1]
%   theta: parameters of Schlogl model (i.e., [k1A, k2, k3B, k4]), [1xd]
%   epsilon: perturbed directions, [Lxd]
%
% ---- OUTPUT PARAMETERS ----
%   RER: numerically-computed relative entropy rate, [1xL]
%   RER_exact: exact relative entropy rate, [1xL]
%   FIM: numerically-computed Fister information matrix, [dxd]
%   FIM_exact: exact Fister information matrix, [dxd]
%
% (c) Math & Stat, Umass, Amherst, 2014

NoJ = 1000000;

StM = [1; -1; 1; -1]; % stoichiometric matrix (dim: #reactions x #species)
rt_tbl = Schlogl_rates_table(Omega, N); % table with the rates of the process

[L, d] = size(epsilon);

RER = zeros(1, L); % relative entropy rate
FIM = zeros(d, d); % Fisher information matrix

T = 0; % time
X_pre = X0; % initial number of molecules


% SSA
for i = 2:NoJ
    rn = rand(2,1); % two uniform random numbers, necessary for SSA
    
    rt = theta .* rt_tbl(X_pre+1, :); % rates of the process
    rt_plus = rt(1)+rt(3);
    rt_minus = rt(2)+rt(4);
%    rt_der = diag(1./theta); % derivatives of the log-rates of the process
    
    tot_rt = sum(rt); % total rate
    trp = cumsum(rt) / tot_rt; % transition probabilities
    
    dt = - log(rn(1))/tot_rt; % advance time
    T = T + dt;

    idx = find(trp>rn(2), 1, 'first'); % find which reaction will occur

    X_new = X_pre + StM(idx); % perform the reaction (i.e., update the number of molecules)
    
    
    % compute RER
    for l = 1:L
        rt_per = (theta+epsilon(l,:)) .* rt_tbl(X_pre+1, :); % rates of the perturbed process
        tot_rt_per = sum(rt_per); % total rate of the perturbed process

        rt_plus_per = rt_per(1)+rt_per(3);
        rt_minus_per = rt_per(2)+rt_per(4);
        
        % compute Relative Entropy Rate
        RER(l) = RER(l) + dt * (rt_plus*log(rt_plus/rt_plus_per) + tot_rt_per - tot_rt);
        if rt_minus~=0
            RER(l) = RER(l) + dt * rt_minus*log(rt_minus/rt_minus_per);
        end

%         % alternative but equivalent numerical approximation of RER
%         if idx==1 || idx==3
%             RER(l) = RER(l) + (log(rt_plus/rt_plus_per) + dt*(tot_rt_per - tot_rt));
%         elseif idx==2 || idx==4
%             RER(l) = RER(l) + (log(rt_minus/rt_minus_per) + dt*(tot_rt_per - tot_rt));
%         end
    end
    
    % compute FIM
    rt_der_plus = [rt_tbl(X_pre+1, 1); 0; rt_tbl(X_pre+1, 3); 0];
    rt_der_minus = [0; rt_tbl(X_pre+1, 2); 0; rt_tbl(X_pre+1, 4)];
    
    FIM = FIM + 0.5 * dt * (rt_der_plus * rt_der_plus') / rt_plus;
    
    if rt_minus~=0
        FIM = FIM + 0.5 *dt *  (rt_der_minus * rt_der_minus') / rt_minus;
    end

    
%     % alternative but equivalent numerical approximation of FIM
%     if idx==1 || idx==3
%         log_rt_der_plus = [rt_tbl(X_pre+1, 1); 0; rt_tbl(X_pre+1, 3); 0] / rt_plus;
%         FIM = FIM + 0.5 * (log_rt_der_plus * log_rt_der_plus');
% 
%     elseif idx==2 || idx==4
%         log_rt_der_minus = [0; rt_tbl(X_pre+1, 2); 0; rt_tbl(X_pre+1, 4)] / rt_minus;
%         FIM = FIM + 0.5 * (log_rt_der_minus * log_rt_der_minus');
%     end
    
    % update
    X_pre = X_new;
end

% normalize
RER = RER / T;
FIM = FIM / T;


% EXACT FORMULAS
n = (0:N)';
c_plus = theta(1)/Omega * n.*(n-1)/2 + theta(3)*Omega;
c_minus = theta(2)/Omega^2 * n.*(n-1).*(n-2)/6 + theta(4)*n;

p_exact = zeros(1, N+1);
p_exact(1)=1;

for i=1:N
    p_exact(i+1) = p_exact(i) * c_plus(i)/c_minus(i+1);
end
p_exact = p_exact / sum(p_exact);

c_plus_per = zeros(N+1,L);
c_minus_per = zeros(N+1,L);
for l = 1:L
    c_plus_per(:,l) = (theta(1)+epsilon(l,1))/Omega * n.*(n-1)/2 + (theta(3)+epsilon(l,3))*Omega;
    c_minus_per(:,l) = (theta(2)+epsilon(l,2))/Omega^2 * n.*(n-1).*(n-2)/6 + (theta(4)+epsilon(l,4))*n;
end


RER_exact = zeros(1, L);
FIM_exact = zeros(d,d);
for i=2:N+1
    for l = 1:L        
        RER_exact(l) = RER_exact(l) + p_exact(i)*(c_plus(i)*log(c_plus(i)/c_plus_per(i,l)));
        RER_exact(l) = RER_exact(l) + p_exact(i)*(c_minus(i)*log(c_minus(i)/c_minus_per(i,l)));
        RER_exact(l) = RER_exact(l) + p_exact(i)*(c_plus_per(i,l)+c_minus_per(i,l)-c_plus(i)-c_minus(i));
    end
    
    
    FIM_exact = FIM_exact + .5*p_exact(i)*[rt_tbl(i,1);0;rt_tbl(i,3);0]*[rt_tbl(i,1),0,rt_tbl(i,3),0] / c_plus(i);
    FIM_exact = FIM_exact + .5*p_exact(i)*[0;rt_tbl(i,2);0;rt_tbl(i,4)]*[0,rt_tbl(i,2),0,rt_tbl(i,4)] / c_minus(i);
end


