function [PFIM, RER] = compute_PFIM_RER_BIO520(data, dname)
%
% Compute PFIM and RER from the stored time-series.
%
%  ---- INPUT PARAMETERS ----
%   dname: directory name of population data
%   data: data structure array
%
%   data.M: number of reactions, [1x1]
%   data.K: number of parameters, [1x1] 
%   data.X: current population of the system, [Nx1]
%	data.T: final time, [1x1]
%   data.cv_glpf: vector of the non-zero elements of the symbolic PFIM, [prop_lenx1]
%   data.prop_index: index of the non-zero elements of the symbolic PFIM, [prop_lenx1]
%   data.con: value of reaction constants, [Kx1]
%   data.stoichio: stoichiometric matrix, [Mx4]
%   data.cm_rea: connectivity matrix for the reactions, [Mx4]
%   data.pert_vec: Matrix of positive and negative 10 percent perturbation for all parameter values, [NoP, K]
%
% ---- OUTPUT PARAMETERS ----
%   PFIM: Fisher Information Matrix, [KxK]
%   RER: a vector of RERs corresponds to the perturbation matrix, [NoPx1]
%
% (c) Math & Stat, Umass, Amherst, 2014


% file names with population data
fnames = dir([dname 'data/bio520/*.mat']);
NoS = length(fnames); % number of simulations


% initialize
[NoV, K] = size(data.pert_vec);

PFIM = zeros(K, K, NoS);
RER = zeros(NoV, NoS);

% compute PFIM and/or RER
fprintf(1, 'Computing PFIM and RER .....\n');
tic;

for i = 1:NoS
    % load reduced jump time and time series 
    load([dname 'data/' fnames(i).name]);
    
    if nargout == 1 % only PFIM is returned
        PFIM(:,:,i) = comp_PFIM(X, data);
    
    elseif nargout == 2 % both PFIM and RER are returned
        PFIM(:,:,i) = comp_PFIM(X, data);
        RER(:,i) = comp_RER(X, data);
    else
        break;
    end
    
    % show the progress
    fprintf(1, '%.1f%% completed in %.2f seconds.\n', i/NoS*100, toc);
end

