function data = initialization_lin(rname)
%
% Initialization for simulation and sensitivity analysis. Reactions, species, reaction rates,
% connectivity and initial configuration are specified. Linear-scale sensitivity analysis
% is considered.
%
%  ---- INPUT PARAMETERS ----
%   rname: file name that describes the model
%
% ---- OUTPUT PARAMETERS ----
%   data.M: number of reactions, [1x1]
%   data.N: number of species, [1x1]
%   data.K: number of parameters, [1x1] 
%   data.X: current population of the system, [Nx1]
%   data.pX: chemical species pattern X(i), [Nx1]
%   data.pC: system parameters pattern con(i), [Kx1]
%   data.theta: name of system parameters, [Kx1]
%   data.prop: cell vector with the propensity functions as MATLAB handles,[Mx1]
%   data.props: cell vector with the propensity functions as strings,[Mx1]
%   data.vprops: cell vector with the propensity functions as strings with numeric coefficients,[Mx1]
%   data.m_glpf: matrix of the gradient of log of propensity functions, [KxK]
%   data.cv_glpf: vector of the non-zero elements of the symbolic PFIM, [prop_lenx1]
%   data.prop_index: index of the non-zero elements of the symbolic PFIM, [prop_lenx1]
%   data.con: value of reaction constants, [Kx1]
%   data.sname: name of chemical species, [Nx1] 
%   data.stoichio: stoichiometric matrix, [Mx4]
%   data.cm_rea: connectivity matrix for the reactions, [Mx4]
%   data.cm_spe: connectivity matrix for the species, [MxK+1]
%
%   Create a get_rate.m file
%   Create a get_PFIM.m file
%
% (c) Math & Stat, Umass, Amherst, 2014


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% load stoichiometric matrix %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
stoichio = load([rname '_stoichiomatrix.txt']);
[M, four] = size(stoichio);

if four ~= 4
    error('FormatChk:stoichioChk', 'Stoichiometric matrix dimension error.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% load reaction propensity functions %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fileID = fopen([rname '_propensities.txt']);
tmp_kr = textscan(fileID, '%s','delimiter','\n');  
fclose(fileID);

[M_tmp, one] = size(tmp_kr{1});

if M_tmp ~= M || one ~= 1
    error('FormatChk:propensityChk','Reaction propensity functions dimension error.');
end

krs=tmp_kr{1};
vkrs=tmp_kr{1};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load reactants and reaction constants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load reactant names and initialize reactants population X
fileID = fopen([rname '_initial_population.txt']);
in_p = textscan(fileID, '%s %s %f');   
fclose(fileID);
X=in_p{3};
rename=in_p{1};

% Load constant names and initialize reaction constants con
fileID = fopen([rname '_reaction_constants.txt']);
r_conf = textscan(fileID, '%s %s %f');   
fclose(fileID);
con=r_conf{3};
cname=r_conf{1};
K=length(cname);

% Create pattern "X(1) X(2) ..."
formatSpec = 'X(%d)';
pX = cell(1 , length(rename));
for i=1:1:length(rename)
	pX(i)=cellstr(sprintf(formatSpec,i));	
end
pX=pX';

% Create pattern "con(1) con(2) ..."
formatSpec = 'con(%d)';
pC = cell(1 , length(cname));
for i=1:1:length(cname)
	pC(i)=cellstr(sprintf(formatSpec,i));	
end
pC=pC';

% Replace all reactant names in propensity functions by "X(i)"
cl= cellfun(@length,rename); % compute length of each cell element
mcl= horzcat(cl,(1:length(rename))'); % Create a matrix of length and index
smcl=sortrows(mcl,1); % Sort row by the length to get order of index 

for i=length(rename):-1:1    % Descending order to avoid replace partials only(X1 and X11)
    ind=smcl(i,2);
	krs=cellfun(@(x) strrep(x,rename(ind),pX(ind)),krs, 'UniformOutput', false);
end

% Replace all constant names in propensity functions by "con(i)"
cl= cellfun(@length,cname); % compute length of each cell element
mcl= horzcat(cl,(1:length(cname))'); % Create a matrix of length and index
smcl=sortrows(mcl,1); % Sort row by the length to get order of index 

for i=length(cname):-1:1
    ind=smcl(i,2);
	krs=cellfun(@(x) strrep(x,cname(ind),pC(ind)),krs, 'UniformOutput', false);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create cell strings containing propensity functions with 
% real valued reactant constants (for MFA)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replace all reactant names in propensity functions by "X(i)"
cl= cellfun(@length,rename); % compute length of each cell element
mcl= horzcat(cl,(1:length(rename))'); % Create a matrix of length and index
smcl=sortrows(mcl,1); % Sort row by the length to get order of index 

for i=length(rename):-1:1    % Descending order to avoid replace partials only(X1 and X11)
    ind=smcl(i,2);
	vkrs=cellfun(@(x) strrep(x,rename(ind),pX(ind)),vkrs, 'UniformOutput', false);
end

% Replace reaction constants in propensity function by their values
cl= cellfun(@length,cname); % compute length of each cell element
mcl= horzcat(cl,(1:length(cname))'); % Create a matrix of length and index
smcl=sortrows(mcl,1); % Sort row by the length to get order of index 

for i=length(cname):-1:1
    ind=smcl(i,2);
	vkrs=cellfun(@(x) strrep(x,cname(ind),num2str(con(ind))),vkrs, 'UniformOutput', false);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define function handle for reaction propensity functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
kr = cell(1 , length(krs));
for j=1:1:length(krs)
	kr{j}=@(X,con) cellfun(@eval, krs{j},'UniformOutput', false);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% load reaction connectivity matrix %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cm_rea = load([rname '_connectivity.txt']);
[M_tmp, four] = size(cm_rea);

if M_tmp ~= M || four ~= 4
    error('FormatChk:connectivityChk', 'Reaction connectivity matrix dimension error.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% load initial population %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fileID = fopen([rname '_initial_population.txt']);
tmp_X = textscan(fileID, '%s %d','delimiter','=');  
fclose(fileID);

[N, one] = size(tmp_X{2});

if one ~= 1
    error('FormatChk:populationChk','Initial population dimension error.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% create species connectivity matrix %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the rows are the species 
% the first column is in total how many reactions react involve that species
% the rest of the columns are the specific reaction which involve the species 

cm_spe = zeros(N, 2*M+1); % "2*M+1" since the specie will appear at most twice(both sides) in M reaction plus first column as total 
no_rea = zeros(N, 1);
for m = 1:M
    for k = 1:4
        if cm_rea(m,k) ~= 0 && cm_rea(m,k) ~= 1    % When not NULL(1) and not empty(0)
            cm_spe(cm_rea(m,k), no_rea(cm_rea(m,k))+2) = m;  % store reaction number          
            no_rea(cm_rea(m,k)) = no_rea(cm_rea(m,k))+1; % store number of occurrence 
        end      
    end
end

cm_spe(:,1) = no_rea;

maxK = max(no_rea);
cm_spe = cm_spe(:,1:maxK+1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the matrix of the gradient of the logarithm of the propensity functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
kr_sym=krs;

% Create pattern "X1 X2 ..."
formatSpec = 'X%d'; % Later only change con's as symbol not X's
spX = cell(1,N);
for i=1:1:N
	spX(i)=cellstr(sprintf(formatSpec,i));	
end
spX=spX';

% Create pattern "con1 con2 ..."
formatSpec = 'con%d';
spC = cell(1,K);
for i=1:1:K
	spC(i)=cellstr(sprintf(formatSpec,i));	
end
spC=spC';

% Create pattern "X(1) X(2) ..."
formatSpec = 'X(%d)'; % Later only change con's as symbol not X's
pX = cell(1,N);
for i=1:1:N
	pX(i)=cellstr(sprintf(formatSpec,i));	
end
pX=pX';

% Create pattern "con(1) con(2) ..."
formatSpec = 'con(%d)';
pC = cell(1,K);
for i=1:1:K
	pC(i)=cellstr(sprintf(formatSpec,i));	
end
pC=pC';

% Replace all reactant names in propensity functions by "Xi"
for i=N:-1:1
    kr_sym=cellfun(@(x) strrep(x,pX(i),spX(i)),kr_sym, 'UniformOutput', true); % If false, then save as cell; if true, save as char
end

% Replace all constant names in propensity functions by "coni"
for i=K:-1:1
    kr_sym=cellfun(@(x) strrep(x,pC(i),spC(i)),kr_sym, 'UniformOutput', true);
end

% Set symbol for xi and coni
sympC = sym(spC);
kr_symb = sym(kr_sym);
v_glpf = sym(zeros(K,1));
m_glpf = sym(zeros(K,K));

% PFIM of Q theta
for i=1:M 
	  for j=1:K 
		  tmp=strcat('log(',kr_sym{i},')'); % add log over propensity function
        sym_glpf=diff(char(tmp),sympC(j),1); % the the gradient of the log of propensity function
        v_glpf(j,1)=sym_glpf;
    end     
    m_glpf = m_glpf+kr_symb(i,1)*(v_glpf*v_glpf.');
end

% Find the non-zero elements of the symbolic matrix
prop_index=find(~(m_glpf==0));
prop_len=length(prop_index);
% Extract and save the non-zero elements of the symbolic matrix
cv_glpf=cell(prop_len,1);
v_glpf=m_glpf(prop_index);

% Change X1, con1 back to X(1) and con(1)
for j=1:prop_len
    cv_glpf(j,1)=cellstr(char(v_glpf(j)));
    for i=length(spC):-1:1
        cv_glpf(j,1)=cellfun(@(x) strrep(x,char(spC(i)),pC(i)),cv_glpf(j,1), 'UniformOutput', false); % change the notation back, i.e. "con1" to "con(1)"
    end   
    for k=length(spX):-1:1
        cv_glpf(j,1)=cellfun(@(x) strrep(x,char(spX(k)),pX(k)),cv_glpf(j,1), 'UniformOutput', false); % change the notation back, i.e. "con1" to "con(1)"
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%
%%% save and return %%%%
%%%%%%%%%%%%%%%%%%%%%%%%
data.M = M;
data.N = N;
data.K = K; 
data.X = X;
data.pX = pX;
data.pC = pC;
data.theta = cname;
data.prop = kr;
data.props = krs;
data.vprops = vkrs;
data.m_glpf = m_glpf;
data.cv_glpf=cv_glpf;
data.prop_index=prop_index;
data.stoichio = stoichio;
data.con = con;
data.sname=rename;
data.cm_rea = cm_rea;
data.cm_spe = cm_spe;

% Create a get_rate.m file
create_getrate_m(data);
% Create a get_PFIM.m file
create_getPFIM_m(data);