function [p_exact, p_approx, p_hist, p_exact_per, p_hist_per] = invMeasure_change(X0, Omega, N, theta, epsilon)
%
% Computes the change of the invariant  measure of the Schlogl model when the
% parameter (i.e., the rates) of the model are perturbed. Three different
% ways were used for the estimation of invariant measure.
%
%  ---- INPUT PARAMETERS ----
%   X0: initial number of molecules, [1x1]
%   Omega: volume of the system, [1x1]
%   N: maximum number of molecules, [1x1]
%   theta: parameters of Schlogl model (i.e., [k1A, k2, k3B, k4]), [1xd]
%   epsilon: perturbed directions, [Lxd]
%
% ---- OUTPUT PARAMETERS ----
%   p_exact: exact invariane measure, [1xN+1]
%   p_approx: Hangii approximation of the invariane measure, [1xN+1]
%   p_hist: numerical (histogram) approximation of the invariane measure, [1xN+1]
%   p_exact_per: exactly-computed perturbed invariane measure, [1xN+1]
%   p_hist_per: numerically-computed perturbed invariane measure, [1xN+1]
%
% (c) Math & Stat, Umass, Amherst, 2014

NoI = 1;

[L, d] = size(epsilon);

p_hist = zeros(1, N+1);
p_exact_per = zeros(L, N+1);
if nargout>4
p_hist_per = zeros(L, N+1);
end

% average
for i = 1:NoI

    [p_exact, p_approx, p_hist_tmp] = Schlogl_invMeasure(X0, Omega, N, theta);

    p_hist = p_hist + p_hist_tmp;
    
    for l = 1:L
        p_exact_per(l,:) = Schlogl_invMeasure(X0, Omega, N, theta+epsilon(l,:));
    end
    
    % restrict to what the numerical method gives
    if nargout>4
    for l = 1:L
        [p_ign1, p_ign1, p_hist_tmp] = Schlogl_invMeasure(X0, Omega, N, theta+epsilon(l,:));
    
        p_hist_per(l,:) = p_hist_per(l,:) + p_hist_tmp;
    end
    end
end

% normalization
p_hist = p_hist/NoI;
if nargout>4
for l = 1:L
    p_hist_per(l,:) = p_hist_per(l,:)/NoI;
end
end


