function output = reaction_cle( data )
%
%   Simulation for well-mixed reaction systems with Chemical Langevin Equation.
%
%  ---- INPUT PARAMETERS ----
%   data: data structure array
%
%   data.M: number of reactions, [1x1]
%   data.N: number of species, [1x1]
%   data.NoJ: number of jumps, [1x1]
%   data.T: final time, [1x1]
%   data.tau: tau leap size,  [1x1]
%   data.X: current population of the system, [Nx1]
%   data.stoichio: stoichiometric matrix, [Mx4]
%   data.cm_rea: connectivity matrix for the reactions, [Mx4]
%	data.con: value of reaction constants, [Kx1]
%
% ---- OUTPUT PARAMETERS ----
%   output.t: time vector, [NoJx1]
%   output.species_ts: the concentrations of each species at each time step, [NoJxN]
%
%  Weilong Hu, 2013, UMass

% net gain of each species after each reaction 
V = zeros(data.N, data.M); 
for m = 1:data.M
    for k = 1:4
        ind = data.cm_rea(m,k);
        if ind ~= 0 && ind ~= 1    % When not NULL(1) and not empty(0)
            V(ind, m) = V(ind, m) +  data.stoichio(m, k);
        end      
    end
end

% initialize output struct
output.species_ts = zeros(data.NoJ, data.N);
output.t          = zeros(data.NoJ, 1);

% initial species population and initial time
output.species_ts(1,:) =  data.X';
output.t(1)            =  0;

% Rates initialization
c = zeros(data.M, 1);
for reaction = 1:data.M
    c(reaction) = reaction_rate(data, reaction);
end

% iterate
for i = 2 : data.NoJ
    % check if rates are NaN
    if(isnan(c))
        error('Rates in simulation became Nan.')
    end

    if(isinf(c))
        error('Rates in simulation became Inf.')
    end
    
    % check if there is nowhere to move
    if(~sum(c))
        fprintf('\n\nSystem reached a degenerate point. Exiting...\n\n'); 
        output.species_ts = output.species_ts(1:i-1,:);
        output.t = output.t(1:i-1);
        return;
    end
    
    % find which reaction will fire
    r = normrnd(0,1,[data.M 1]);   %  Array of random numbers from standard normal distribution
    dW = sqrt(data.tau)*r;

    % update species after reaction
    data.X = double(data.X) + V*c*data.tau + V*(sqrt(c).*dW);

    % If population is negative, change it to 0
    data.X(data.X<0)=0;
    
    % update rates 
    for reaction = 1:data.M
            c = update_rates(c, data, reaction);
    end
    
    % update output
    output.species_ts(i,:)  =  data.X';
    output.t(i)             =  output.t(i-1) + data.tau;
    
    if output.t(i) > data.T
        output.species_ts = output.species_ts(1:i,:);
        output.t = output.t(1:i);
        break;
    end
end

