function output = reaction_mfa(data)
%
%   Mean Field Approximation (MFA) simulation for well-mixed reaction systems.
%
%  ---- INPUT PARAMETERS ----
%   data: data structure array
%
%   data.M: number of reactions, [1x1]
%   data.N: number of species, [1x1]
%   data.T: final time, [1x1]
%   data.X: current population of the system, [Nx1]
%   data.stoichio: stoichiometric matrix, [Mx4]
%   data.cm_rea: connectivity matrix for the reactions, [Mx4]
%	data.vprops: cell vector with the propensity functions as strings with numeric coefficients,[Mx1]
%
% ---- OUTPUT PARAMETERS ----
%   output.t: time vector, [NoSx1]
%   output.species_ts: the concentrations of each species at each time step, [NoSxN]
%
% (c) Math & Stat, Umass, Amherst, 2014

stoichio = data.stoichio;
X = data.X;
% create the expression strings
dy(data.N).expr = [];

for m = 1:data.M
    idx1 = data.cm_rea(m, 1);
    idx2 = data.cm_rea(m, 2);
    idx3 = data.cm_rea(m, 3);
    idx4 = data.cm_rea(m, 4);
    rate = char(data.vprops{m});
    % add to/subtract from each species the corresponding concentration
    if idx1 % if there is a first reactant
        if idx2 % if there is a second reactant
            % first reactant
            dy(idx1).expr = [dy(idx1).expr ' - (' rate ')'];

            % second reactant
            dy(idx2).expr = [dy(idx2).expr ' - (' rate ')'];

            % first product
            if idx3
                if idx3 == idx1
                    dy(idx3).expr = [dy(idx3).expr ' + ' num2str((stoichio(m,1)+stoichio(m,3)+1)) '* (' rate ')'];
                elseif idx3 == idx2
                    dy(idx3).expr = [dy(idx3).expr ' + ' num2str((stoichio(m,2)+stoichio(m,3)+1)) '* (' rate ')'];
                else
                    dy(idx3).expr = [dy(idx3).expr ' + (' rate ')'];
                end
            end
            % second product
            if idx4
                if idx4 == idx1
                    dy(idx4).expr = [dy(idx4).expr ' + ' num2str((stoichio(m,1)+stoichio(m,4)+1)) '* (' rate ')'];
                elseif idx4 == idx2
                    dy(idx4).expr = [dy(idx4).expr ' + ' num2str((stoichio(m,2)+stoichio(m,4)+1)) '* (' rate ')'];
                else
                    dy(idx4).expr = [dy(idx4).expr ' + (' rate ')'];
                end
            end                

        else % if there is no second reactant
            % first reactant
            dy(idx1).expr = [dy(idx1).expr ' - (' rate ')'];

            % first product
            if idx3
                if idx3 == idx1
                    dy(idx3).expr = [dy(idx3).expr ' + ' num2str((stoichio(m,1)+stoichio(m,3)+1)) '* (' rate ')'];
                else
                    dy(idx3).expr = [dy(idx3).expr ' +  (' rate ')'];
                end
            end
            % second product
            if idx4
                if idx4 == idx1
                    dy(idx4).expr = [dy(idx4).expr ' + ' num2str((stoichio(m,1)+stoichio(m,4)+1)) '* (' rate ')'];
                else
                    dy(idx4).expr = [dy(idx4).expr ' +  (' rate ')'];
                end
            end
        end
    else % if there is no first reactant, there is no second reactant as well                      
        % first product
        if idx3
            dy(idx3).expr = [dy(idx3).expr ' +  (' rate ')'];
        end
        % second product
        if idx4
            dy(idx4).expr = [dy(idx4).expr ' +  (' rate ')'];            
        end
    end
end

% create the function
str = strtrim(sprintf(';%s' ,dy(2:data.N).expr));
str_f = ['@(t,X) [0' str '];'];

dy = str2func(str_f); % make string to function

% solve the ODE system
options = odeset('RelTol', 1e-5); % provide the specifications

[t, y] = ode15s( dy, [0, data.T], double(X), options);

output.t = t;
output.species_ts = y;



