function output = reaction_ssa(data)
%
% Stochastic simulation algorithm (SSA) for well-mixed reaction systems.
%
%  ---- INPUT PARAMETERS ----
%   data: data structure array
%
%   data.M: number of reactions, [1x1]
%   data.N: number of species, [1x1]
%   data.NoJ: number of jumps, [1x1]
%   data.T: final time, [1x1]
%   data.X: current population of the system, [Nx1]
%   data.stoichio: stoichiometric matrix, [Mx4]
%   data.cm_rea: connectivity matrix for the reactions, [Mx4]
%   data.cm_spe: connectivity matrix for the species, [MxK+1]
%	data.con: value of reaction constants, [Kx1]
%
% ---- OUTPUT PARAMETERS ----
%   output.t: time vector, [NoJx1]
%   output.species_ts: the concentrations of each species at each time step, [NoJxN]
%
%
% (c) Math & Stat, Umass, Amherst, 2014

% initialize output struct
output.species_ts = zeros(data.NoJ, data.N);
output.t = zeros(data.NoJ, 1);

output.species_ts(1,:) = data.X';
output.t(1) = 0;

% Rates initialization
c = zeros(data.M, 1);
for k = 1:data.M
    c(k) = reaction_rate(data, k);
end
c_cum = cumsum(c);
c0 = c_cum(end);


% iterate
for i = 2 : data.NoJ
    % check if there is nowhere to move
    if(~c0)
        fprintf('\n\nSystem reached a degenerate point. Exiting...\n\n'); 
        % list the current rates
        fprintf('\n Current reaction rates are %1.20e\n',c); 
        % list the current populations
        fprintf('\n Current populations are %1.20e\n',output.species_ts(i-1,:)); 
        output.species_ts = output.species_ts(1:i-1,:);
        output.t = output.t(1:i-1);
        return;
    end
    
    % find which reaction will fire  % find the first index of the c_cum bigger than c0*rand 
    reaction = find( c_cum>c0*rand, 1, 'first');   % "rand" gives a random value from Unif Dist between (0,1)
    
    % compute waiting time
    dt = - log(rand)/c0;  % !!!!! Noted this rand is independent to the previous one !!!!!!!!
    
    % check if there is a problem in the chosen reaction i.e if no element in c_cum bigger than c0*rand 
    if(~reaction)
       fprintf('\n\nError in %s function. 0 reaction was chosen. Exiting...\n\n', mfilename);
       return;
    end
       
    % update species after reaction
    for k=1:4
        ind = data.cm_rea(reaction, k);
        if data.cm_rea(reaction, k)
            data.X(ind) = data.X(ind) + data.stoichio(reaction, k);
        end
    end
    
    % update rates involving species of 'reaction' after species update
    c = update_rates(c, data, reaction);
    c_cum = cumsum(c);
    c0 = c_cum(end);
    
    % update output
    output.species_ts(i,:) = data.X';
    output.t(i) = output.t(i-1) + dt;
    
    if output.t(i) > data.T
        output.species_ts = output.species_ts(1:i,:);
        output.t = output.t(1:i);
        break;
    end
end

