function rxn2csm(fname)
%
% Read three files with the reactions mechanism, reaction constants and initial populations 
% to creates the connectivity matrix, stoichiometric matrix and the propensity 
% functions. Tested for MATLAB versions 2013a. 
%
%***********************************************************************************************************
%
% Purpose: Given three text files with the reactions mechanism, reaction constants and initial populations, it  
% generates 3 text files containing stoichiometric data, connectivity matrix, and propensity functions. 
%
%************************************************************************************************************
%
% BEFORE YOU USE
% Note:
%      1. Reactions with only one or two species in each direction can be treated. 
%      2. A Species name cannot be two separate words - for example 
%			carbon + oxygen = carbon dioxide is incorrect
%	        carbon + oxygen = carbon_dioxide is ok
%      4. The first species is always the NULL which describes the birth or the death of species
%
%
%************************************************************************************************************
%
% Example: Consider a reaction mechanism involving 3 reactants (X1, X2 and X3).
%          The reaction constants for the three reactions are k1, k2 and k3. 
%          X1 + X2 --> X3  k1*X1*X2
%          X3 --> X1 + X2  k2*X3
%          X2 --> NULL     k3*X2  
%
%-----------------------------------------------------------------------------------------
%
% Input files: 
%          fname: directory with file name, which is used to load the following 3 .txt files
%          1. reactants.txt: It lists the reactions with the propensity functions, i.e.,
%                            X1 + X2 = X3  'k1*X1*X2'
%                            X3 = X1 + X2  'k2*X3'
%                            X2 = NULL     'k3*X2'
%          2. initial_population.txt: It assigns numbers to each reactant, i.e.,
%                            NULL = 0
%                            X1 = 100
%                            X2 = 100
%                            X3 = 100  
%          3. reaction_constants: It assigns numbers to each reactant constant, i.e.,
%                            k1 = 62849
%                            k2 = 0.70599
%                            k3 = 1 
%
% Remark: all the elements in the input files should be separated by spaces.
%------------------------------------------------------------------------------------------
%
% Output files:
%          1. stoichiometry.txt: The stoichiometry of participating species in reactions is generated in the 
%                              following format: 
%                             Stoich_1   Stoich_2    Stoich_3   Stoich_4
%                                 -1        -1          1          0
%                                 -1        -0          1          1
%                                 -1        -0          0          0
%          2. connectivity.txt: The connectivity is generated in the following format
%                             and shows which species are involved in each reaction : 
%                             Connectivity_1 Connectivity_2 Connectivity_3 Connectivity_4
%                i.e.,              2              3              4              0
%                                   4              0              2              3
%                                   3              0              1              0
%          3. propensities.txt: The reaction propensity functions are generated in the following format: 
%                i.e.,        k1*X1*X2
%                             k2*X3
%                             k3*X2
%--------------------------------------------------------------------------------------------------
%
% (c) Math & Stat, Umass, Amherst, 2014

disp('Creating data structures from the reaction system...');
% take input file name until the dot
outputfile = fname(1: regexp(fname,'\.')-1);

%*************************************************
% #1 Initial population
%*************************************************

%*************************************************
% #1.1 Check initial_pop file 
%*************************************************
fileID = fopen([outputfile '_initial_population.txt']); 
in_p = textscan(fileID, '%s','delimiter','\n');  
fclose(fileID);

lines=in_p{1};
n = length(lines);

for i=1:1:n
	%**************************************
	% #1.1.1 Check and split line by " = "
	%**************************************
	
	% Check if "=" is missing		
	if isempty(strfind(lines{i}, '='))		
		error('FormatChk:eqChk', 'No equal sign in line %d.', i)
	end

	% Check number of "=" is 1
	if length(strfind(lines{i}, '='))~=1
		error('FormatChk:eqChk', 'Wrong number of equal sign in line %d.', i)
	end	
	
	% Split line by "=" and remove all while spaces
    s=lines{i};
    bline1=s(1:strfind(s,'=')-1);
    bline2=s((strfind(s,'=')+1):length(s));
    bline1=sscanf(bline1,'%s');
    bline2=sscanf(bline2,'%s');
    
	%************************************************
	% #1.1.2 Check reactants and initial populations
	%************************************************
	
	% Check if reactant is empty
    if isempty(bline1) %#ok<ALIGN>
		error('FormatChk:inPopChk', 'Blank reactant in line %d.', i)
	end	

	% Check if initial population is empty
	if isempty(bline2)
		error('FormatChk:inPopChk', 'Blank initial population in line %d.', i)
	end	
end

%*************************************************
% #1.2 Load initial_pop file 
%*************************************************
fileID = fopen([outputfile '_initial_population.txt']);
in_p = textscan(fileID, '%s %s %d');   
fclose(fileID);

reactants=in_p{1};

%*****************************************************
% #1.3 Evaluate initial population
%*****************************************************
ini_pop = textread([outputfile '_initial_population.txt'],'%s','delimiter', '\n');
ini_popf=cellstr(ini_pop);
for i=1:1:length(ini_pop)
	evalc(ini_popf{i,1});
end;

%*****************************************************
% #2 Reaction constants
%*****************************************************
%*************************************************
% #2.1 Check re_con file 
%*************************************************
fileID = fopen([outputfile '_reaction_constants.txt']); 
r_con = textscan(fileID, '%s','delimiter','\n');  
fclose(fileID);

lines=r_con{1};
n = length(lines);

for i=1:1:n
	%**************************************
	% #2.1.1 Check and split line by " = "
	%**************************************
	
	% Check if "=" is missing		
	if isempty(strfind(lines{i}, '='))		
		error('FormatChk:eqChk', 'No equal sign in line %d.', i)
	end

	% Check number of "=" is 1
	if length(strfind(lines{i}, '='))~=1
		error('FormatChk:eqChk', 'Wrong number of equal sign in line %d.', i)
	end	
	
	% Split line by "=" and remove all while spaces
    s=lines{i};
    bline1=s(1:strfind(s,'=')-1);
    bline2=s((strfind(s,'=')+1):length(s));
    bline1=sscanf(bline1,'%s');
    bline2=sscanf(bline2,'%s');

	%******************************************************
	% #1.1.2 Check reaction constant name and constant
	%******************************************************
	
	% Check if constant name is empty
    if isempty(bline1) %#ok<ALIGN>
		error('FormatChk:inPopChk', 'Blank reactant in line %d.', i)
	end	

	% Check if initial population is empty
	if isempty(bline2)
		error('FormatChk:inPopChk', 'Blank initial population in line %d.', i)
	end	
end

%*****************************************************
% #2.2 Evaluate reaction constants
%*****************************************************
r_con = textread([outputfile '_reaction_constants.txt'],'%s','delimiter', '\n');
r_conf=cellstr(r_con);
for i=1:1:length(r_con)
	evalc(r_conf{i,1});
end;

%*************************************************
% #3 Reactions and rate functions
%*************************************************

%****************************************************
% #3.1 Load reactions and rate functions line by line
%****************************************************
fileID = fopen(fname); 
react = textscan(fileID, '%s','delimiter','\n');  
fclose(fileID);

%*************************************************
% #3.2 Declare and initialize variables
%*************************************************
lines=react{1};
n = length(lines);
coeff=ones(n,4);
Connectivity=zeros(n,4);
prop=cell(n,1);

for i=1:1:n

	%**************************************
	% #3.3 Check format line by line
	%**************************************
	
	%**************************************
	% #3.3.1 Check and split line by " ' "
	%**************************************
	
	% Check if all " ' " are missing		
	if isempty(strfind(lines{i}, ''''))		
		error('FormatChk:quoteChk', 'No rate function in line %d.', i)
	end

	% Check number of " ' " is two
	if length(strfind(lines{i}, ''''))~=2
		error('FormatChk:quoteChk', 'Wrong number of quotations in line %d.', i)
	end	
	
	% Split line by " ' " and remove all while spaces
    s=lines{i};
    index=strfind(s,'''');
    bline1=s(1:index(1)-1);
    bline2=s((index(1)+1):index(2)-1);
    bline1=sscanf(bline1,'%s');
    bline2=sscanf(bline2,'%s');
    
	%*********************************
	% #3.3.2 Check rate functions
	%*********************************
	
	% Check if rate function is empty
	if isempty(bline2)
		error('FormatChk:rateChk', 'Blank rate function in line %d.', i)
	end	

	% Check if all components are defined
	try
	   evalc(bline2);
	catch exception
	   error('FormatChk:rateChk','Undefined variables in rate function in line %d.', i)
	end
	%************************************************
	% #3.3.3 Set up reaction and propensity function
	%************************************************
	str=bline1;
	prop(i)=cellstr(bline2);

	%********************************
	% #3.3.4 Check reaction format
	%********************************
	
	% Check and split by "="
	if isempty(strfind(str, '='))		
		error('FormatChk:reactionChk', 'No equal sign in the reaction in line %d.', i)
	end	
	
	if length(strfind(str, '='))~=1
		error('FormatChk:reactionChk', 'Multiple equal signs in the reaction in line %d.', i)
	end	
	
% 	str2=strsplit(str,'=');
    s=str;
    str21=s(1:strfind(s,'=')-1);
    str22=s((strfind(s,'=')+1):length(s));
    
	% Check and split by "+"
	% Set inputs: U and C
	if ~isempty(strfind(str21, '+'))	
		if length(strfind(str21, '+'))~=1 %#ok<ALIGN>
			error('FormatChk:reactionChk', 'More than two input reactants in line %d.', i)
        end	

        str211=str21(1:strfind(str21,'+')-1);
        str212=str21((strfind(str21,'+')+1):length(str21));
        
		if (~isempty(str211))&&(~isempty(str212))
			U=str211;
			C=str212;
		else
			error('FormatChk:reactionChk', 'Missing input reactants in line %d.', i)
		end
	else 
		U=str21;
		C=cellstr(' ');
	end	
	% Set outputs: P and M
	if ~isempty(strfind(str22, '+'))	 %#ok<ALIGN>
		if length(strfind(str22, '+'))~=1 %#ok<ALIGN>
			error('FormatChk:reactionChk', 'More than two input reactants in line %d.', i)
        end
		
        str221=str22(1:strfind(str22,'+')-1);
        str222=str22((strfind(str22,'+')+1):length(str22));
        
		if (~isempty(str221))&&(~isempty(str222))
			P=str221;
			M=str222;
		else
			error('FormatChk:reactionChk', 'Missing output reactants in line %d.', i)
		end
	else 
		P=str22;
		M=cellstr(' ');
    end	
    
	%*************************************************
	% #3.4 Extract coefficients 	
	%*************************************************
	U1=char(U);
	bool_l=isletter(U1);
	ind_l=find(bool_l,1,'first');
	if isempty(U1(1:(ind_l-1)))
		coeff(i,1)=1;
	else
		coeff(i,1)=str2double(U1(1:(ind_l-1)));
	end
	U1=U1(ind_l:end);
	U=cellstr(U1);
	
	C1=char(C);
	bool_l=isletter(C1);
	ind_l=find(bool_l,1,'first');	
	if isempty(C1(1:(ind_l-1)))
		coeff(i,2)=1;
	else
		coeff(i,2)=str2double(C1(1:(ind_l-1)));
	end
	C1=C1(ind_l:end);
	C=cellstr(C1);
	
	P1=char(P);
	bool_l=isletter(P1);
	ind_l=find(bool_l,1,'first');
	if isempty(P1(1:(ind_l-1)))
		coeff(i,3)=1;
	else
		coeff(i,3)=str2double(P1(1:(ind_l-1)));
	end
	P1=P1(ind_l:end);
	P=cellstr(P1);
	
	M1=char(M);
	bool_l=isletter(M1);
	ind_l=find(bool_l,1,'first');
	if isempty(M1(1:(ind_l-1)))
		coeff(i,4)=1;
	else
		coeff(i,4)=str2double(M1(1:(ind_l-1)));
	end
	M1=M1(ind_l:end);
	M=cellstr(M1);
	
	clear U1 C1 P1 M1;

	%*************************************************
	% #3.5 Set up connectivity matrix 
	%*************************************************
	for j=1:1:length(reactants) %#ok<ALIGN>
        if ~isempty(U)
            if strcmp(U,reactants(j))
                Connectivity(i,1)=j;
            end
        end
        if ~isempty(C)
            if strcmp(C,reactants(j))
                Connectivity(i,2)=j;
            end
        end
        if ~isempty(P)
            if strcmp(P,reactants(j))
                Connectivity(i,3)=j;
            end
        end        
        if ~isempty(M)
            if strcmp(M,reactants(j))
                Connectivity(i,4)=j;
            end
        end

    end	

	%*************************************************
	% #3.6 Set up stoichiometric matrix
	%*************************************************
	Stoichio = coeff;
	Stoichio(Connectivity==0 | Connectivity==1) = 0;
	Stoichio(:,1:2) = -Stoichio(:,1:2);

end 

%*************************************************
% #4. Write data in files
%*************************************************
%*************************************************
% #4.1 Write propensity functions 
%*************************************************
fid = fopen([outputfile '_propensities.txt'],'w'); 
for i=1:1:length(prop)
    g=char(prop{i});
    fprintf(fid,'%s\n',g);     
end
fclose(fid);

%*************************************************
% #4.2 Write connectivity and stoichiometry matrix 
%*************************************************
dlmwrite([outputfile '_connectivity.txt']  , Connectivity,' '); 
dlmwrite([outputfile '_stoichiomatrix.txt'], Stoichio,' ');
