function sbml2txt(file, outputfile)
%
% Remove the 'listOfModifiers' tag from xml file and rewrite if necessary.
% Load xml file into matlab and create three input files: reactions
% mechanism, reaction constants and initial populations.
%
%  ---- INPUT PARAMETERS ----
%   file: complete path of the location of xml file 
%   outputfile: name of the output file
% ---- OUTPUT PARAMETERS ----
%   reactions mechanism
%   reaction constants
%   initial populations
%
% (c) Math & Stat, Umass, Amherst, 2014

% Load xml file
xmlDoc=xmlread(file);
xRoot = xmlDoc.getDocumentElement;
allListItems = xRoot.getElementsByTagName('reaction');

% Remove the 'listOfModifiers' tag from xml file if necessary 
for i=0:allListItems.getLength-1
    thisItem = allListItems.item(i);  
    rChd = xRoot.getElementsByTagName('listOfModifiers');
    if rChd.item(0)~=[]    
        thisItem.removeChild(rChd.item(0));
    end
end

% Overwrite the xml file
xmlwrite(file,xmlDoc);

% Import xml file into matlab
BIO=sbmlimport(file);

% Create reactions cellstring
r_array = sbioselect(BIO,'Type','reaction');
reactions=cell(size(r_array));
revId=zeros(size(r_array));
for index = 1:size(r_array,1)    
    if(isempty(strfind(r_array(index).Reaction,'<')))
        reactions(index,1) = cellstr(strrep(r_array(index).Reaction,'->','='));
    else
        reactions(index,1) = cellstr(strrep(r_array(index).Reaction,'<->','='));
        revId(index,1)=1;
    end
end

% Create the reversed reactions if both-sided
if(any(revId))
    rev=reactions(logical(revId));
    newrev=rev;
    for i=1:size(rev,1)
        eqindex=cell2mat(strfind(rev(i,1),'='));
        currev=rev(i,1);
        str=char(currev);
        newrev(i,1)=cellstr([str((eqindex+1):length(str)),' = ',str(1:eqindex-1)]);
    end
end

% Combine reactions and reversed reactions
if(any(revId))
    freactions=[reactions; newrev];
else
    freactions=reactions;
end

% Create propensity functions
propFunc=cell(size(r_array));
for index = 1:size(r_array,1)    
    propFunc(index,1) = cellstr(r_array(index).ReactionRate);
end
if(any(revId))
    rev=propFunc(logical(revId));
end
% Combine propensity and propensity
if(any(revId))
    fpropFunc=[propFunc; rev];
else
    fpropFunc=propFunc;
end

% Create initial_population cellstring
s_array = sbioselect(BIO,'Type','species');
initial_population=cell(size(s_array));
for index = 1:size(s_array,1)
    initial_population(index,1) = cellstr([s_array(index).Name,' = ',num2str(s_array(index).InitialAmount)]);
end
initial_population=[cellstr('NULL = 0'); initial_population];

% Create reaction_constants cellstring
p_array = sbioselect(BIO,'Type','parameter');
reaction_constants=cell(size(p_array));
for index = 1:size(p_array,1)
    reaction_constants(index,1) = cellstr([p_array(index).Name,' = ',num2str(p_array(index).Value)]);
end

% Write reaction and propensity to txt
fileID = fopen([outputfile '.txt'],'w');
formatSpec = '%s  ''%s''\n';
for row = 1:size(freactions,1)
fprintf(fileID,formatSpec,freactions{row,:},fpropFunc{row,:});
end
fclose(fileID);

% Write initial_population to txt
fileID = fopen([outputfile '_initial_population.txt'],'w');
formatSpec = '%s\n';
for row = 1:size(initial_population,1)
fprintf(fileID,formatSpec,initial_population{row,:});
end
fclose(fileID);

% Write reaction_constants to txt
fileID = fopen([outputfile '_reaction_constants.txt'],'w');
formatSpec = '%s\n';
for row = 1:size(reaction_constants,1)
fprintf(fileID,formatSpec,reaction_constants{row,:});
end
fclose(fileID);